@@ -32,7 +32,13 @@ func (c *baseCommand) initConfig(filename string, optional bool) (*configpkg.Con
 			return nil, nil
 		}
 
-		return nil, errors.New("A Waypoint configuration file (waypoint.hcl) is required but wasn't found.")
+		return nil, errors.New(
+			"A Waypoint configuration file (waypoint.hcl) is required but wasn't found.\n" +
+				"The command can use configuration stored on the server by adding project/app\n" +
+				"as a final argument on the command line.\n" +
+				"For instance: `waypoint deployment list webapp/web` for the app 'web' in\n" +
+				"the project 'webapp'",
+		)
 	}
 
 	return c.initConfigLoad(path)
@@ -71,7 +77,10 @@ func (c *baseCommand) initConfigLoad(path string) (*configpkg.Config, error) {
 //
 // If ctx is nil, c.Ctx will be used. If ctx is non-nil, that context will be
 // used and c.Ctx will be ignored.
-func (c *baseCommand) initClient(ctx context.Context) (*clientpkg.Project, error) {
+func (c *baseCommand) initClient(
+	ctx context.Context,
+	connectOpts ...serverclient.ConnectOption,
+) (*clientpkg.Project, error) {
 	// We use our flag-based connection info if the user set an addr.
 	var flagConnection *clicontext.Config
 	if v := c.flagConnection; v.Server.Address != "" {
@@ -82,11 +91,12 @@ func (c *baseCommand) initClient(ctx context.Context) (*clientpkg.Project, error
 	// the following precedence: (1) context (2) env (3) flags where the
 	// later values override the former.
 	var err error
-	connectOpts := []serverclient.ConnectOption{
+	connectOpts = append([]serverclient.ConnectOption{
 		serverclient.FromContext(c.contextStorage, ""),
 		serverclient.FromEnv(),
 		serverclient.FromContextConfig(flagConnection),
-	}
+		serverclient.Logger(c.Log.Named("serverclient")),
+	}, connectOpts...)
 	c.clientContext, err = serverclient.ContextConfig(connectOpts...)
 	if err != nil {
 		return nil, err
