@@ -83,9 +83,19 @@ func (c *InstallCommand) Run(args []string) int {
 		return 1
 	}
 
+	// collect any args after a `--` break to pass forward as secondary flags
+	var secondaryArgs []string
+	for i, f := range args {
+		if f == "--" {
+			secondaryArgs = args[(i + 1):]
+			break
+		}
+	}
+
 	result, err := p.Install(ctx, &serverinstall.InstallOpts{
-		Log: log,
-		UI:  c.ui,
+		Log:            log,
+		UI:             c.ui,
+		ServerRunFlags: secondaryArgs,
 	})
 	if err != nil {
 		c.ui.Output(
@@ -333,7 +343,7 @@ func (c *InstallCommand) Flags() *flag.Sets {
 		f.BoolVar(&flag.BoolVar{
 			Name:    "runner",
 			Target:  &c.flagRunner,
-			Usage:   "Install a runner in addition to the server",
+			Usage:   "Install a runner in addition to the server.",
 			Default: true,
 			Hidden:  true,
 		})
@@ -389,6 +399,14 @@ Alias: waypoint install
   URL service by manually running the server. If you disable the URL service,
   you do not need to accept any terms.
 
+  To further customize the server installation, you may pass advanced flag options
+  specified in the documentation for the 'server run' command. To set these values,
+  include a '--' after the full argument list for 'install', followed by these
+  advanced flag options. As an example, to set the server log level to trace
+  and disable the UI, the command would be:
+
+    waypoint install -platform=docker -accept-tos -- -vvv -disable-ui
+
 ` + c.Flags().Help())
 }
 
@@ -459,6 +477,30 @@ func installRunner(
 	}
 	s.Done()
 
+	// If this installation platform supports an out-of-the-box ODR
+	// config then we set that up. This enables on-demand runners to
+	// work immediately.
+	if odc, ok := p.(serverinstall.OnDemandRunnerConfigProvider); ok {
+		s = sg.Add("Registering on-demand runner configuration...")
+
+		odr := odc.OnDemandRunnerConfig()
+		if odr != nil {
+			_, err = client.UpsertOnDemandRunnerConfig(ctx, &pb.UpsertOnDemandRunnerConfigRequest{
+				Config: odr,
+			})
+
+			if err != nil {
+				s.Update("Error creating ondemand runner: %s", err)
+			} else {
+				s.Update("Registered ondemand runner!")
+			}
+		} else {
+			s.Update("Install type did not provide an ondemand runner config")
+		}
+
+		s.Done()
+	}
+
 	return 0
 }
 
