@@ -23,6 +23,7 @@ type ReleaseCreateCommand struct {
 	flagDeployment  string
 	flagPrune       bool
 	flagPruneRetain int
+	flagId          idFormat
 }
 
 func (c *ReleaseCreateCommand) Run(args []string) int {
@@ -32,13 +33,17 @@ func (c *ReleaseCreateCommand) Run(args []string) int {
 	if err := c.Init(
 		WithArgs(args),
 		WithFlags(c.Flags()),
-		WithSingleApp(),
+		WithMultipleApp(),
 	); err != nil {
 		return 1
 	}
 
 	client := c.project.Client()
 	err := c.DoApp(c.Ctx, func(ctx context.Context, app *clientpkg.App) error {
+		// UI -- this should happen at the top so that the app name shows clearly
+		// for any errors we may encounter prior to the actual release op
+		app.UI.Output("Releasing %s...", app.Ref().Application, terminal.WithHeaderStyle())
+
 		// Get the latest release
 		release, err := client.GetLatestRelease(ctx, &pb.GetLatestReleaseRequest{
 			Application: app.Ref(),
@@ -114,7 +119,7 @@ func (c *ReleaseCreateCommand) Run(args []string) int {
 			} else {
 				c.Log.Warn("deployment already released")
 				app.UI.Output(strings.TrimSpace(releaseUpToDate),
-					deploy.Id,
+					c.flagId.FormatId(deploy.Sequence, deploy.Id),
 					terminal.WithSuccessStyle())
 				return nil
 			}
@@ -126,9 +131,6 @@ func (c *ReleaseCreateCommand) Run(args []string) int {
 			return ErrSentinel
 		}
 
-		// UI
-		app.UI.Output("Releasing...", terminal.WithHeaderStyle())
-
 		// Release
 		result, err := app.Release(ctx, &pb.Job_ReleaseOp{
 			Deployment: deploy,
@@ -233,12 +235,12 @@ func (c *ReleaseCreateCommand) Synopsis() string {
 
 func (c *ReleaseCreateCommand) Help() string {
 	return formatHelp(`
-Usage: waypoint release [options] [id]
+Usage: waypoint release [options]
 
   Open a deployment to traffic.
 
   This defaults to the latest deployment. Other deployments can be
-  specified by sequence number or long ID.
+  specified by ID using the '-deployment' flag.
 
 ` + c.Flags().Help())
 }
