@@ -39,7 +39,7 @@ func (c *ReleaseListCommand) Run(args []string) int {
 	if err := c.Init(
 		WithArgs(args),
 		WithFlags(c.Flags()),
-		WithSingleApp(),
+		WithMultipleApp(),
 	); err != nil {
 		return 1
 	}
@@ -48,6 +48,13 @@ func (c *ReleaseListCommand) Run(args []string) int {
 	client := c.project.Client()
 
 	err := c.DoApp(c.Ctx, func(ctx context.Context, app *clientpkg.App) error {
+		if !c.flagJson {
+			// UI -- this should happen at the top so that the app name shows clearly
+			// for any errors we may encounter prior to the actual table output
+			// but we also don't want to corrupt the json
+			app.UI.Output("%s", app.Ref().Application, terminal.WithHeaderStyle())
+		}
+
 		var wsRef *pb.Ref_Workspace
 		if !c.flagWorkspaceAll {
 			wsRef = c.project.WorkspaceRef()
@@ -125,7 +132,7 @@ func (c *ReleaseListCommand) Run(args []string) int {
 					status = "✔"
 					statusColor = terminal.Green
 
-					if resp.Releases[0] != nil {
+					if resp.Releases[0] != nil && resp.Releases[0].Release.Id == b.Id {
 						status = "🚀"
 					}
 
