@@ -2,14 +2,18 @@ package core
 
 import (
 	"context"
+	"errors"
 
 	"github.com/golang/protobuf/proto"
 	"github.com/golang/protobuf/ptypes/any"
 	"github.com/hashicorp/go-argmapper"
 	"github.com/hashicorp/go-hclog"
+	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/status"
 
 	"github.com/hashicorp/waypoint-plugin-sdk/component"
 	"github.com/hashicorp/waypoint/internal/config"
+	"github.com/hashicorp/waypoint/internal/plugin"
 	pb "github.com/hashicorp/waypoint/internal/server/gen"
 )
 
@@ -32,9 +36,24 @@ func (a *App) Build(ctx context.Context, optFuncs ...BuildOption) (
 	}
 	defer c.Close()
 
+	cr, err := componentCreatorMap[component.RegistryType].Create(ctx, a, nil)
+	if err != nil {
+		if status.Code(err) == codes.Unimplemented {
+			cr = nil
+			err = nil
+		} else {
+			return nil, nil, err
+		}
+	}
+
+	if cr != nil {
+		defer cr.Close()
+	}
+
 	// First we do the build
 	_, msg, err := a.doOperation(ctx, a.logger.Named("build"), &buildOperation{
 		Component:   c,
+		Registry:    cr,
 		HasRegistry: a.config.RegistryUse() != "",
 	})
 	if err != nil {
@@ -92,6 +111,7 @@ func (opts *buildOptions) Validate() error {
 // buildOperation implements the operation interface.
 type buildOperation struct {
 	Component *Component
+	Registry  *Component
 	Build     *pb.Build
 
 	HasRegistry bool
@@ -129,12 +149,42 @@ func (op *buildOperation) Upsert(
 }
 
 func (op *buildOperation) Do(ctx context.Context, log hclog.Logger, app *App, _ proto.Message) (interface{}, error) {
+	args := []argmapper.Arg{
+		argmapper.Named("HasRegistry", op.HasRegistry),
+	}
+
+	// If there is a registry defined and it implements RegistryAccess...
+	if op.Registry != nil {
+		if ra, ok := op.Registry.Value.(component.RegistryAccess); ok {
+			raw, err := app.callDynamicFunc(ctx, log, nil, op.Component, ra.AccessInfoFunc())
+			if err == nil {
+				args = append(args, argmapper.Typed(raw))
+
+				if pm, ok := raw.(interface {
+					TypedAny() *any.Any
+				}); ok {
+					any := pm.TypedAny()
+
+					// ... which we make available to build plugin.
+					args = append(args, plugin.ArgNamedAny("access_info", any))
+					log.Debug("injected access info")
+				} else {
+					log.Error("unexpected response type from callDynamicFunc", "type", hclog.Fmt("%T", raw))
+					return nil, errors.New("AccessInfoFunc didn't provide a typed any")
+				}
+			} else {
+				log.Error("error calling dynamic func", "error", err)
+				return nil, err
+			}
+		}
+	}
+
 	return app.callDynamicFunc(ctx,
 		log,
 		(*component.Artifact)(nil),
 		op.Component,
 		op.Component.Value.(component.Builder).BuildFunc(),
-		argmapper.Named("HasRegistry", op.HasRegistry),
+		args...,
 	)
 }
 
@@ -142,13 +192,13 @@ func (op *buildOperation) StatusPtr(msg proto.Message) **pb.Status {
 	return &(msg.(*pb.Build).Status)
 }
 
-func (op *buildOperation) ValuePtr(msg proto.Message) **any.Any {
+func (op *buildOperation) ValuePtr(msg proto.Message) (**any.Any, *string) {
 	v := msg.(*pb.Build)
 	if v.Artifact == nil {
 		v.Artifact = &pb.Artifact{}
 	}
 
-	return &v.Artifact.Artifact
+	return &v.Artifact.Artifact, &v.Artifact.ArtifactJson
 }
 
 var _ operation = (*buildOperation)(nil)
