@@ -3,7 +3,6 @@ package singleprocess
 import (
 	"context"
 	"errors"
-	"fmt"
 	"sync"
 	"sync/atomic"
 	"testing"
@@ -469,121 +468,6 @@ func TestApplicationPollHandler(t *testing.T) {
 	require.NoError(err)
 	client := server.TestServer(t, impl)
 
-	// Create a project with an application
-	respProj, err := client.UpsertProject(ctx, &pb.UpsertProjectRequest{
-		Project: serverptypes.TestProject(t, &pb.Project{
-			Name: "Example",
-			DataSource: &pb.Job_DataSource{
-				Source: &pb.Job_DataSource_Local{
-					Local: &pb.Job_Local{},
-				},
-			},
-			DataSourcePoll: &pb.Project_Poll{
-				Enabled:  true,
-				Interval: "15ms",
-			},
-			StatusReportPoll: &pb.Project_AppStatusPoll{
-				Enabled:  false,
-				Interval: "15ms",
-			},
-			Applications: []*pb.Application{
-				{
-					Project: &pb.Ref_Project{Project: "Example"},
-					Name:    "apple-app",
-				},
-			},
-		}),
-	})
-	require.NoError(err)
-	project := respProj.Project
-
-	// Grab next poll time
-	state := testServiceImpl(impl).state
-	a, _, err := state.ApplicationPollPeek(nil)
-	require.NoError(err)
-	require.Nil(a) // Apps Next Poll should be 0 since not started yet
-
-	// Wait a bit. The interval is so low that this should trigger
-	// multiple loops through the poller. But we want to ensure we
-	// have only one poll job queued.
-	time.Sleep(50 * time.Millisecond)
-
-	// Do a deployment
-	resp, err := client.UpsertDeployment(ctx, &pb.UpsertDeploymentRequest{
-		Deployment: serverptypes.TestValidDeployment(t, &pb.Deployment{
-			Component: &pb.Component{
-				Name: "testapp",
-			},
-			Application: &pb.Ref_Application{
-				Application: "apple-app",
-				Project:     "Example",
-			},
-		}),
-	})
-	require.NoError(err)
-	require.NotNil(resp)
-
-	// Update the app to start polling
-	project.StatusReportPoll = &pb.Project_AppStatusPoll{
-		Enabled: true,
-	}
-	_, err = client.UpsertProject(ctx, &pb.UpsertProjectRequest{
-		Project: project,
-	})
-	require.NoError(err)
-
-	// App poll time should be set
-	a, pollTime, err := state.ApplicationPollPeek(nil)
-	require.NoError(err)
-	require.NotNil(pollTime)
-	require.NotNil(a) // Apps Next Poll should be set
-
-	// Wait a bit. The interval is so low that this should trigger
-	// multiple loops through the poller. But we want to ensure we
-	// have only one poll job queued.
-	time.Sleep(50 * time.Millisecond)
-
-	// Check for our condition, we do eventually here because if we're
-	// in a slow environment then this may still be empty.
-	require.Eventually(func() bool {
-		// We should have a single poll job
-		var jobs []*pb.Job
-		raw, err := testServiceImpl(impl).state.JobList()
-		for _, j := range raw {
-			if j.State != pb.Job_ERROR {
-				jobs = append(jobs, j)
-			}
-		}
-
-		if err != nil {
-			t.Logf("err: %s", err)
-			return false
-		}
-
-		return len(jobs) == 1
-	}, 5*time.Second, 50*time.Millisecond)
-
-	// Cancel our poller to ensure it stops
-	testServiceImpl(impl).Close()
-
-	// ensure the next poll is after the initial poll before waiting
-	// next poll time gets set when a app poll is marked complete
-	a, nextPollTime, err := state.ApplicationPollPeek(nil)
-	require.NoError(err)
-	require.NotNil(a)
-	require.NotNil(nextPollTime)
-	require.True(nextPollTime.After(pollTime))
-}
-
-func TestApplicationPollHandler_turnoff(t *testing.T) {
-	require := require.New(t)
-	ctx := context.Background()
-
-	// Create our server
-	impl, err := New(WithDB(testDB(t)))
-	require.NoError(err)
-	client := server.TestServer(t, impl)
-
 	appName := "apple-app"
 
 	// Create a project with an application
@@ -595,10 +479,6 @@ func TestApplicationPollHandler_turnoff(t *testing.T) {
 					Local: &pb.Job_Local{},
 				},
 			},
-			DataSourcePoll: &pb.Project_Poll{
-				Enabled:  true,
-				Interval: "15ms",
-			},
 			StatusReportPoll: &pb.Project_AppStatusPoll{
 				Enabled:  false,
 				Interval: "15ms",
@@ -665,10 +545,129 @@ func TestApplicationPollHandler_turnoff(t *testing.T) {
 	require.Eventually(func() bool {
 		// We should have a single poll job
 		var jobs []*pb.Job
+
+		raw, err := testServiceImpl(impl).state.JobList()
+		for _, j := range raw {
+			if j.State != pb.Job_ERROR && j.SingletonId == appStatusPollSingletonId(appName, appStatusPollOperationTypeDeployment) {
+				jobs = append(jobs, j)
+			}
+		}
+
+		if err != nil {
+			t.Logf("err: %s", err)
+			return false
+		}
+
+		return len(jobs) == 1
+	}, 5*time.Second, 50*time.Millisecond)
+
+	// Cancel our poller to ensure it stops
+	testServiceImpl(impl).Close()
+
+	// ensure the next poll is after the initial poll before waiting
+	// next poll time gets set when a app poll is marked complete
+	a, nextPollTime, err := state.ApplicationPollPeek(nil)
+	require.NoError(err)
+	require.NotNil(a)
+	require.NotNil(nextPollTime)
+	require.True(nextPollTime.After(pollTime))
+}
+
+func TestApplicationPollHandler_fullLifecycle(t *testing.T) {
+	require := require.New(t)
+	ctx := context.Background()
+
+	// Create our server
+	impl, err := New(WithDB(testDB(t)))
+	require.NoError(err)
+	client := server.TestServer(t, impl)
+
+	appName := "apple-app"
+
+	// Create a project with an application
+	respProj, err := client.UpsertProject(ctx, &pb.UpsertProjectRequest{
+		Project: serverptypes.TestProject(t, &pb.Project{
+			Name: "Example",
+			DataSource: &pb.Job_DataSource{
+				Source: &pb.Job_DataSource_Local{
+					Local: &pb.Job_Local{},
+				},
+			},
+			//DataSourcePoll: &pb.Project_Poll{
+			//	Enabled:  true,
+			//	Interval: "15ms",
+			//},
+			StatusReportPoll: &pb.Project_AppStatusPoll{
+				Enabled:  false,
+				Interval: "15ms",
+			},
+			Applications: []*pb.Application{
+				{
+					Project: &pb.Ref_Project{Project: "Example"},
+					Name:    appName,
+				},
+			},
+		}),
+	})
+	require.NoError(err)
+	project := respProj.Project
+
+	// Grab next poll time
+	state := testServiceImpl(impl).state
+	a, _, err := state.ApplicationPollPeek(nil)
+	require.NoError(err)
+	require.Nil(a) // Apps Next Poll should be 0 since not started yet
+
+	// Wait a bit. The interval is so low that this should trigger
+	// multiple loops through the poller. But we want to ensure we
+	// have only one poll job queued.
+	time.Sleep(50 * time.Millisecond)
+
+	// Do a deployment
+	deployResp, err := client.UpsertDeployment(ctx, &pb.UpsertDeploymentRequest{
+		Deployment: serverptypes.TestValidDeployment(t, &pb.Deployment{
+			Component: &pb.Component{
+				Name: "testapp",
+			},
+			Application: &pb.Ref_Application{
+				Application: appName,
+				Project:     "Example",
+			},
+		}),
+	})
+	require.NoError(err)
+	require.NotNil(deployResp)
+
+	// Update the app to start polling
+	project.StatusReportPoll = &pb.Project_AppStatusPoll{
+		Enabled:  true,
+		Interval: "15ms",
+	}
+	_, err = client.UpsertProject(ctx, &pb.UpsertProjectRequest{
+		Project: project,
+	})
+	require.NoError(err)
+
+	// App poll time should be set
+	a, pollTime, err := state.ApplicationPollPeek(nil)
+	require.NoError(err)
+	require.NotNil(pollTime)
+	require.NotNil(a) // Apps Next Poll should be set
+
+	// Wait a bit. The interval is so low that this should trigger
+	// multiple loops through the poller. But we want to ensure we
+	// have only one poll job queued.
+	time.Sleep(50 * time.Millisecond)
+
+	// Check for our condition, we do eventually here because if we're
+	// in a slow environment then this may still be empty.
+	require.Eventually(func() bool {
+		// We should have a single poll job for just the deployment
+		var jobs []*pb.Job
 		raw, err := testServiceImpl(impl).state.JobList()
 		for _, j := range raw {
 			if j.State != pb.Job_ERROR &&
-				j.SingletonId == fmt.Sprintf("appl-poll/%s", appName) {
+				j.SingletonId == appStatusPollSingletonId(appName, appStatusPollOperationTypeDeployment) {
 				// App status polling should only have this singleton id
 				jobs = append(jobs, j)
 			}
@@ -682,6 +681,56 @@ func TestApplicationPollHandler_turnoff(t *testing.T) {
 		return len(jobs) == 1
 	}, 5*time.Second, 50*time.Millisecond)
 
+	// Do a release
+	releaseResp, err := client.UpsertRelease(ctx, &pb.UpsertReleaseRequest{
+		Release: serverptypes.TestValidRelease(t, &pb.Release{
+			Component: &pb.Component{
+				Name: "testapp",
+			},
+			Application: &pb.Ref_Application{
+				Application: appName,
+				Project:     "Example",
+			},
+		}),
+	})
+	require.NoError(err)
+	require.NotNil(releaseResp)
+
+	// Wait a bit. The interval is so low that this should trigger
+	// multiple loops through the poller. But we want to ensure we
+	// have only one poll job queued.
+	time.Sleep(50 * time.Millisecond)
+
+	// Make sure we're polling on two status reports
+
+	// Check for our condition, we do eventually here because if we're
+	// in a slow environment then this may still be empty.
+	require.Eventually(func() bool {
+		// We should have a poll job for a deployment and another for a release
+
+		raw, err := testServiceImpl(impl).state.JobList()
+		releaseJobs := 0
+		deployJobs := 0
+		for _, j := range raw {
+			t.Logf("Found job in state %s with id %s", j.State, j.SingletonId)
+			if j.State != pb.Job_ERROR {
+				if j.SingletonId == appStatusPollSingletonId(appName, appStatusPollOperationTypeRelease) {
+					releaseJobs++
+				}
+				if j.SingletonId == appStatusPollSingletonId(appName, appStatusPollOperationTypeDeployment) {
+					deployJobs++
+				}
+			}
+		}
+
+		if err != nil {
+			t.Logf("err: %s", err)
+			return false
+		}
+
+		return releaseJobs == 1 && deployJobs == 1
+	}, 5*time.Second, 50*time.Millisecond)
+
 	// Update the app to stop polling
 	project.StatusReportPoll = &pb.Project_AppStatusPoll{
 		Enabled: false,
