@@ -236,8 +236,13 @@ func TestJobAssign(t *testing.T) {
 			Id: "A",
 		})))
 
+		// Should get a peeked job
+		job, err := s.JobPeekForRunner(context.Background(), &pb.Runner{Id: "R_A"})
+		require.NoError(err)
+		require.NotNil(job)
+
 		// Assign it, we should get this build
-		job, err := s.JobAssignForRunner(context.Background(), &pb.Runner{Id: "R_A"})
+		job, err = s.JobAssignForRunner(context.Background(), &pb.Runner{Id: "R_A"})
 		require.NoError(err)
 		require.NotNil(job)
 		require.Equal("A", job.Id)
@@ -253,6 +258,11 @@ func TestJobAssign(t *testing.T) {
 		require.Error(err)
 		require.Nil(job)
 		require.Equal(ctx.Err(), err)
+
+		// Should not block if requested
+		job, err = s.JobPeekForRunner(context.Background(), &pb.Runner{Id: "R_A"})
+		require.NoError(err)
+		require.Nil(job)
 	})
 
 	t.Run("blocking on any", func(t *testing.T) {
@@ -353,6 +363,14 @@ func TestJobAssign(t *testing.T) {
 			require.Equal("A", job.Id)
 		}
 
+		// Peeking still returns blocked jobs. We do this because otherwise
+		// its possible for peek to return empty when there is an eligible job,
+		// its just waiting.
+		job, err := s.JobPeekForRunner(context.Background(), &pb.Runner{Id: "R_A"})
+		require.NoError(err)
+		require.NotNil(job)
+		require.Equal("B", job.Id)
+
 		// Get the next value in a goroutine
 		{
 			ctx, cancel := context.WithCancel(context.Background())
