@@ -44,6 +44,14 @@ export default Factory.extend({
     name: 'kubernetes',
   }),
 
+  'kubernetes-apply': trait({
+    name: 'kubernetes-apply',
+  }),
+
+  'aws-ecr': trait({
+    name: 'aws-ecr',
+  }),
+
   'with-random-name': trait({
     afterCreate(component) {
       component.update('name', randomNameForType(component.type));
