@@ -2,6 +2,7 @@ import { ListReleasesRequest, ListReleasesResponse, GetReleaseRequest } from 'wa
 import { Request, Response } from 'ember-cli-mirage';
 import { decode } from '../helpers/protobufs';
 
+// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types, @typescript-eslint/no-explicit-any
 export function list(schema: any, { requestBody }: Request): Response {
   let requestMsg = decode(ListReleasesRequest, requestBody);
   let projectName = requestMsg.getApplication().getProject();
@@ -21,7 +22,8 @@ export function list(schema: any, { requestBody }: Request): Response {
   return this.serialize(resp, 'application');
 }
 
-export function get(schema: any, { requestBody }: Request) {
+// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types, @typescript-eslint/no-explicit-any
+export function get(schema: any, { requestBody }: Request): Response {
   let requestMsg = decode(GetReleaseRequest, requestBody);
   let id = requestMsg.getRef().getId();
   let model = schema.releases.find(id);
